import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Timer;
import java.util.TimerTask;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;

public class UE20Aufgabe20 {

	public static void main(String[] args) {
		Clock clock = new Clock();
		ClockFrame frame = new ClockFrame(clock);
		frame.setVisible(true);
	}

}

class ClockFrame extends JFrame {

	private Clock clock;
	private JLabel display;

	/**
	 * ffnet ein Fenster mit einer Uhr-Anzeige
	 */
	public ClockFrame(Clock clock) {
		super("Stoppuhr");
		this.clock = clock;
		
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		this.display = new JLabel(this.clock.toString(), SwingConstants.CENTER);
		JButton startButton = new JButton("start");
		JButton stopButton = new JButton("stop");
		stopButton.setEnabled(false);
		JButton resetButton = new JButton("reset");

		StartAction startAction = new StartAction(startButton, stopButton,
				this.clock, this);
		StopAction stopAction = new StopAction(startButton, stopButton,
				startAction);
		ResetAction resetAction = new ResetAction(startButton, stopButton,
				this.clock, this, startAction);

		startButton.addActionListener(startAction);
		stopButton.addActionListener(stopAction);
		resetButton.addActionListener(resetAction);

		this.setLayout(new GridLayout(2, 1));
		this.add(this.display);

		JPanel controlPanel = new JPanel();
		controlPanel.setLayout(new GridLayout(1, 3));
		controlPanel.add(startButton);
		controlPanel.add(stopButton);
		controlPanel.add(resetButton);

		this.add(controlPanel);
		this.pack();
	}

	/**
	 * aktualisiert die von der Uhr angezeigte Uhrzeit
	 */
	public void update() {
		this.display.setText(this.clock.toString());
	}

}

class StartAction implements ActionListener {

	private JButton startButton;
	private JButton stopButton;
	private Clock clock;
	private ClockFrame frame;
	private Timer clockTimer;

	public StartAction(JButton startButton, JButton stopButton, Clock clock,
			ClockFrame frame) {
		this.startButton = startButton;
		this.stopButton = stopButton;
		this.clock = clock;
		this.frame = frame;
		this.clockTimer = null;
	}

	public Timer getClockTimer() {
		return this.clockTimer;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		this.clockTimer = new Timer();
		this.clockTimer.scheduleAtFixedRate(new ClockTask(this.clock,
				this.frame), 1000, 1000);
		this.startButton.setEnabled(false);
		this.stopButton.setEnabled(true);
	}
}

class StopAction implements ActionListener {

	private JButton startButton;
	private JButton stopButton;
	private StartAction startAction;

	public StopAction(JButton startButton, JButton stopButton,
			StartAction startAction) {
		this.startButton = startButton;
		this.stopButton = stopButton;
		this.startAction = startAction;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		this.startAction.getClockTimer().cancel();
		this.startButton.setEnabled(true);
		this.stopButton.setEnabled(false);
	}
}

class ResetAction implements ActionListener {

	private JButton startButton;
	private JButton stopButton;
	private Clock clock;
	private ClockFrame frame;
	private StartAction startAction;

	public ResetAction(JButton startButton, JButton stopButton, Clock clock,
			ClockFrame frame, StartAction startAction) {
		this.startButton = startButton;
		this.stopButton = stopButton;
		this.clock = clock;
		this.frame = frame;
		this.startAction = startAction;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if (this.startAction.getClockTimer() != null) {
			this.startAction.getClockTimer().cancel();
		}
		this.clock.reset();
		this.frame.update();
		this.startButton.setEnabled(true);
		this.stopButton.setEnabled(false);
	}
}

class ClockTask extends TimerTask {

	private Clock clock;
	private ClockFrame frame;

	public ClockTask(Clock clock, ClockFrame frame) {
		this.clock = clock;
		this.frame = frame;
	}

	@Override
	public void run() {
		this.clock.addSeconds(1);
		this.frame.update();
	}
}