import java.util.ArrayList;

interface LRStackI {

	// liefert genau dann true, wenn sich kein Element auf dem LRStack
	// befindet
	public boolean isEmpty();

	// fuegt den uebergebenen Wert auf der rechten Seite des LRStacks an
	public void pushRight(int value);

	// entfernt das am weitesten rechts liegende Element aus dem LRStack
	// und liefert es als Methodenwert; ist der LRStack leer, wird 0 geliefert
	public int popRight();

	// fuegt den uebergebenen Wert auf der linken Seite des LRStacks an
	public void pushLeft(int value);

	// entfernt das am weitesten links liegende Element aus dem LRStack
	// und liefert es als Methodenwert; ist der LRStack leer, wird 0 geliefert
	public int popLeft();

	// liefert einen String, in dem die Werte der Elemente des LRStack
	// von links nach rechts enthalten sind, nach jedem Wert steht ein
	// Leerzeichen; Beispiel: "12 2 -34 "
	public String toString();

	// liefert einen neuen LRStack, der dieselben Werte in derselben
	// Reihenfolge wie der aufgerufene LRStack speichert; Achtung: Die
	// beiden LRStacks sollen unabhaengig voneinander sein, d.h. sie
	// sollen sich keine Elemente teilen!
	public LRStackI clone();

	// liefert genau dann true; wenn beide LRStacks dieselben Werte in
	// derselben Reihenfolge speichern
	public boolean equals(Object obj);
}

class LRStack implements LRStackI {

	protected ArrayList<Integer> elems;

	public LRStack() {
		elems = new ArrayList<Integer>();
	}

	public LRStack(LRStack stack) {
		for (int value : stack.elems) {
			elems.add(value);
		}
	}

	@Override
	public LRStackI clone() {
		return new LRStack(this);
	}

	public boolean isEmpty() {
		return elems.isEmpty();
	}

	public void pushRight(int value) {
		elems.add(value);
	}

	public int popRight() {
		if (elems.isEmpty()) {
			return 0;
		}
		return elems.remove(elems.size() - 1);
	}

	public void pushLeft(int value) {
		elems.add(0, value);
	}

	public int popLeft() {
		if (elems.isEmpty()) {
			return 0;
		}
		return elems.remove(0);
	}

	@Override
	public String toString() {
		String result = "";
		for (int i = 0; i < elems.size(); i++) {
			result += elems.get(i) + " ";
		}
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		LRStack other = (LRStack) obj;
		if (elems == null) {
			if (other.elems != null) {
				return false;
			}
		} else if (!elems.equals(other.elems)) {
			return false;
		}
		return true;
	}

	@Override
	// nicht gefordert
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((elems == null) ? 0 : elems.hashCode());
		return result;
	}
}

public class UE20Aufgabe35 {

	public static void main(String[] args) {
		LRStack stack = new LRStack();
		stack.pushLeft(23);
		stack.pushLeft(-34);
		stack.pushRight(5);
		stack.pushLeft(-8);
		stack.pushRight(67);
		System.out.println(stack.toString());
		stack.popLeft();
		stack.popRight();
		System.out.println(stack.toString());
	}

}
